## upload package
Truy cập https://pypi.org/account/register/
Đăng ký tài khoản và xác thực email.

(xoa existing venv)
rm -rf venv

sudo apt install python3.12-venv
source venv/bin/activate
pip install build twine
(trong thu muc chua pyproject.toml)
pip install build
python -m build

Kết quả: thư mục dist/ sẽ có file .whl và .tar.gz

upload len PyPI
twine upload dist/*

## user install
python3 -m venv venv
source venv/bin/activate
pip install codeguard-taikt

## user setup mcp.json
"servers": {
  "codeguard": {
    "command": "/duong_dan_toi_venv/bin/codeguard",
    "args": ["--transport", "stdio"]
  }
}



Để triển khai MCP server bằng PyInstaller, bạn cần thực hiện các bước sau:

---

### 1. Sửa code MCP server để đọc file tài nguyên (markdown) đúng cách

Giả sử bạn đang đọc file markdown như sau:
```python
with open("copilot_md/Custom.md", "r") as f:
    data = f.read()
```

Bạn cần thay bằng đoạn sau ở mọi nơi đọc file tài nguyên:

```python
import sys
import os

def resource_path(relative_path):
    if hasattr(sys, '_MEIPASS'):
        return os.path.join(sys._MEIPASS, relative_path)
    return os.path.join(os.path.abspath("."), relative_path)

# Đọc file markdown
with open(resource_path("copilot_md/Custom.md"), "r") as f:
    data = f.read()
```

- Nếu có nhiều file, dùng hàm này cho mọi file tài nguyên đóng gói.

---

### 2. Build file thực thi độc lập bằng PyInstaller

Kích hoạt venv (nếu dùng), cài PyInstaller:
```bash
pip install pyinstaller
```

Build file thực thi, đóng gói các file markdown:
```bash
pyinstaller --onefile lgedv/server.py --name codeguard \
  --add-data "copilot_md/Custom.md:copilot_md" \
  --add-data "copilot_md/Misracpp2008Guidelines_en.md:copilot_md" \
  --add-data "copilot_md/CertcppGuidelines_en.md:copilot_md"
```
- Thêm tất cả file markdown cần thiết bằng `--add-data`.

File thực thi sẽ nằm ở `dist/codeguard`.

---

### 3. Đóng gói và upload lên internet

- Nén file thực thi và các tài liệu hướng dẫn thành file `.zip` hoặc `.tar.gz`.
- Upload lên server, Google Drive, hoặc dịch vụ chia sẻ file (hoặc trang web của bạn).

---

### 4. Hướng dẫn người dùng tải về, cài đặt và sử dụng

**Bước 1:** Tải file thực thi về máy (giải nén nếu cần).

**Bước 2:** Chạy trực tiếp file thực thi:
```bash
./codeguard --transport stdio
```
- Không cần cài pip, không cần môi trường ảo, không cần source code.

**Bước 3:** Cấu hình đường dẫn trong `mcp.json`:
```json
"lgedv": {
  "command": "/duong_dan_toi_file/codeguard",
  "args": ["--transport", "stdio"],
  "env": {
    "CPP_DIR": "/duong_dan_toi_cpp_dir",
    "CUSTOM_RULE_PATH": "copilot_md/Custom.md"
  }
}
```
- Đường dẫn file markdown là đường dẫn tương đối hoặc tuyệt đối, tùy cách bạn đóng gói.

---

### 5. Lưu ý

- Build trên Ubuntu 18.04 để đảm bảo chạy được trên nhiều bản Linux mới hơn.
- Nếu file markdown cần chỉnh sửa, hướng dẫn user copy vào đúng vị trí hoặc sửa code để nhận đường dẫn từ biến môi trường.

---

Nếu bạn cần ví dụ sửa code cụ thể cho file nào trong MCP server, hãy chỉ rõ file đó!


Để triển khai đồng thời cả hai cách (pip package và PyInstaller + VSCode extension) mà không phải thay đổi nhiều code MCP server, bạn nên:

### 1. Thiết kế code lấy đường dẫn file rule linh hoạt

Trong file `config.py`, sửa như sau:

```python
def get_rule_path(rule_name, default_relative_path):
    # Ưu tiên lấy từ biến môi trường (cho extension/pyinstaller)
    env_var = f"{rule_name.upper()}_RULE_PATH"
    env_path = os.environ.get(env_var)
    if env_path:
        return env_path
    # Nếu không có, dùng đường dẫn mặc định (cho pip package)
    return os.path.join(BASE_DIR, default_relative_path)

RULE_PATHS = {
    "misra": get_rule_path("misra", "resources/Misracpp2008Guidelines_en.md"),
    "lgedv": get_rule_path("lgedv", "resources/LGEDVRuleGuide.md"),
    "certcpp": get_rule_path("certcpp", "resources/CertcppGuidelines_en.md"),
    "critical": get_rule_path("critical", "resources/CriticalRuleGuideLines.md"),
    "rapidScan": get_rule_path("rapidScan", "resources/RapidScanGuidelines_en.md"),
    "custom": os.environ.get("CUSTOM_RULE_PATH", os.path.join(BASE_DIR, "resources/CustomRule.md"))
}
```

- Khi chạy qua pip: các file rule nằm trong package, code sẽ dùng đường dẫn mặc định.
- Khi chạy qua extension/PyInstaller: truyền đường dẫn file rule qua biến môi trường, code sẽ ưu tiên lấy từ đó.

### 2. Khi build bằng PyInstaller

- Đóng gói file rule vào binary như hướng dẫn trước.
- Extension truyền đường dẫn file rule qua biến môi trường khi start server.

### 3. Khi phân phối qua pip

- Đóng gói file rule vào package (dùng package_data hoặc include trong pyproject.toml).
- Code sẽ tự động lấy file rule từ package.

---

**Tóm lại:**  
- Sử dụng hàm lấy đường dẫn ưu tiên biến môi trường, fallback về đường dẫn mặc định.
- Không cần thay đổi logic đọc file, chỉ cần truyền đúng đường dẫn khi dùng extension/PyInstaller.

Cách này giúp bạn giữ code MCP server gần như nguyên vẹn, dễ bảo trì, dễ triển khai cả hai phương án!